@@ -3,17 +3,18 @@
  * SPDX-License-Identifier: Apache-2.0.
  */
 
+#include <aws/common/array_list.h>
 #include <aws/event-stream/event_stream.h>
 #include <aws/testing/aws_test_harness.h>
 
 struct test_decoder_data {
     struct aws_event_stream_message_prelude latest_prelude;
-    char latest_header_name[100];
-    char latest_header_value[100];
+    struct aws_array_list headers_list;
     uint8_t *latest_payload;
     size_t written;
     struct aws_allocator *alloc;
     int latest_error;
+    uint32_t message_crc;
 };
 
 static void s_decoder_test_on_payload_segment(
@@ -60,10 +61,16 @@ static void s_decoder_test_header_received(
     (void)decoder;
     (void)prelude;
     struct test_decoder_data *decoder_data = (struct test_decoder_data *)user_data;
-    memset(decoder_data->latest_header_name, 0, sizeof(decoder_data->latest_header_name));
-    memcpy(decoder_data->latest_header_name, header->header_name, (size_t)header->header_name_len);
-    memset(decoder_data->latest_header_value, 0, sizeof(decoder_data->latest_header_value));
-    memcpy(decoder_data->latest_header_value, header->header_value.variable_len_val, header->header_value_len);
+    aws_event_stream_add_header(&decoder_data->headers_list, header);
+}
+
+static void s_decoder_test_on_complete(
+    struct aws_event_stream_streaming_decoder *decoder,
+    uint32_t message_crc,
+    void *user_data) {
+    (void)decoder;
+    struct test_decoder_data *decoder_data = (struct test_decoder_data *)user_data;
+    decoder_data->message_crc = message_crc;
 }
 
 static void s_decoder_test_on_error(
@@ -103,15 +110,16 @@ static int s_test_streaming_decoder_incoming_no_op_valid_single_message_fn(struc
     (void)ctx;
     struct test_decoder_data decoder_data = {.latest_payload = 0, .written = 0, .alloc = allocator, .latest_error = 0};
 
+    struct aws_event_stream_streaming_decoder_options decoder_options = {
+        .on_payload_segment = s_decoder_test_on_payload_segment,
+        .on_prelude = s_decoder_test_on_prelude_received,
+        .on_header = s_decoder_test_header_received,
+        .on_complete = s_decoder_test_on_complete,
+        .on_error = s_decoder_test_on_error,
+        .user_data = &decoder_data};
+
     struct aws_event_stream_streaming_decoder decoder;
-    aws_event_stream_streaming_decoder_init(
-        &decoder,
-        allocator,
-        s_decoder_test_on_payload_segment,
-        s_decoder_test_on_prelude_received,
-        s_decoder_test_header_received,
-        s_decoder_test_on_error,
-        &decoder_data);
+    aws_event_stream_streaming_decoder_init_from_options(&decoder, allocator, &decoder_options);
 
     struct aws_byte_buf test_buf = aws_byte_buf_from_array(test_data, sizeof(test_data));
     ASSERT_SUCCESS(
@@ -126,6 +134,7 @@ static int s_test_streaming_decoder_incoming_no_op_valid_single_message_fn(struc
     if (decoder_data.latest_payload) {
         aws_mem_release(allocator, decoder_data.latest_payload);
     }
+    ASSERT_UINT_EQUALS(0x7D98C8FF, decoder_data.message_crc);
 
     aws_event_stream_streaming_decoder_clean_up(&decoder);
 
@@ -145,15 +154,16 @@ static int s_test_streaming_decoder_incoming_application_no_headers_fn(struct aw
     (void)ctx;
     struct test_decoder_data decoder_data = {.latest_payload = 0, .written = 0, .alloc = allocator, .latest_error = 0};
 
+    struct aws_event_stream_streaming_decoder_options decoder_options = {
+        .on_payload_segment = s_decoder_test_on_payload_segment,
+        .on_prelude = s_decoder_test_on_prelude_received,
+        .on_header = s_decoder_test_header_received,
+        .on_complete = s_decoder_test_on_complete,
+        .on_error = s_decoder_test_on_error,
+        .user_data = &decoder_data};
+
     struct aws_event_stream_streaming_decoder decoder;
-    aws_event_stream_streaming_decoder_init(
-        &decoder,
-        allocator,
-        s_decoder_test_on_payload_segment,
-        s_decoder_test_on_prelude_received,
-        s_decoder_test_header_received,
-        s_decoder_test_on_error,
-        &decoder_data);
+    aws_event_stream_streaming_decoder_init_from_options(&decoder, allocator, &decoder_options);
 
     struct aws_byte_buf test_buf = aws_byte_buf_from_array(test_data, sizeof(test_data));
 
@@ -182,6 +192,7 @@ static int s_test_streaming_decoder_incoming_application_no_headers_fn(struct aw
     if (decoder_data.latest_payload) {
         aws_mem_release(allocator, decoder_data.latest_payload);
     }
+    ASSERT_UINT_EQUALS(0xC3653936, decoder_data.message_crc);
 
     aws_event_stream_streaming_decoder_clean_up(&decoder);
 
@@ -209,16 +220,18 @@ static int s_test_streaming_decoder_incoming_application_one_compressed_header_p
         .alloc = allocator,
         .latest_error = 0,
     };
+    aws_event_stream_headers_list_init(&decoder_data.headers_list, allocator);
+
+    struct aws_event_stream_streaming_decoder_options decoder_options = {
+        .on_payload_segment = s_decoder_test_on_payload_segment,
+        .on_prelude = s_decoder_test_on_prelude_received,
+        .on_header = s_decoder_test_header_received,
+        .on_complete = s_decoder_test_on_complete,
+        .on_error = s_decoder_test_on_error,
+        .user_data = &decoder_data};
 
     struct aws_event_stream_streaming_decoder decoder;
-    aws_event_stream_streaming_decoder_init(
-        &decoder,
-        allocator,
-        s_decoder_test_on_payload_segment,
-        s_decoder_test_on_prelude_received,
-        s_decoder_test_header_received,
-        s_decoder_test_on_error,
-        &decoder_data);
+    aws_event_stream_streaming_decoder_init_from_options(&decoder, allocator, &decoder_options);
 
     struct aws_byte_buf test_buf = aws_byte_buf_from_array(test_data, sizeof(test_data));
 
@@ -233,18 +246,22 @@ static int s_test_streaming_decoder_incoming_application_one_compressed_header_p
     const char *content_type = "content-type";
     const char *content_type_value = "application/json";
 
+    struct aws_event_stream_header_value_pair latest_header;
+    aws_array_list_get_at(&decoder_data.headers_list, &latest_header, 0);
+    struct aws_byte_buf latest_header_value = aws_event_stream_header_value_as_string(&latest_header);
+
     ASSERT_BIN_ARRAYS_EQUALS(
         content_type,
         strlen(content_type),
-        decoder_data.latest_header_name,
-        strlen(decoder_data.latest_header_name),
+        latest_header.header_name,
+        latest_header.header_name_len,
         "header name should have been %s",
         content_type);
     ASSERT_BIN_ARRAYS_EQUALS(
         content_type_value,
         strlen(content_type_value),
-        decoder_data.latest_header_value,
-        strlen(decoder_data.latest_header_value),
+        latest_header_value.buffer,
+        latest_header_value.len,
         "header value should have been %s",
         content_type_value);
 
@@ -265,7 +282,9 @@ static int s_test_streaming_decoder_incoming_application_one_compressed_header_p
     if (decoder_data.latest_payload) {
         aws_mem_release(allocator, decoder_data.latest_payload);
     }
+    ASSERT_UINT_EQUALS(0x8D9C08B1, decoder_data.message_crc);
 
+    aws_event_stream_headers_list_cleanup(&decoder_data.headers_list);
     return 0;
 }
 
@@ -273,6 +292,146 @@ AWS_TEST_CASE(
     test_streaming_decoder_incoming_application_one_compressed_header_pair_valid,
     s_test_streaming_decoder_incoming_application_one_compressed_header_pair_valid_fn)
 
+static int s_test_streaming_decoder_incoming_application_one_int32_header_pair_valid_fn(
+    struct aws_allocator *allocator,
+    void *ctx) {
+    (void)ctx;
+    /* clang-format off */
+    uint8_t test_data[] = {
+        0x00, 0x00, 0x00, 0x1b,         /* total length */
+        0x00, 0x00, 0x00, 0x0b,         /* headers length */
+        0xe5, 0xc0, 0xa0, 0x72,         /* prelude crc */
+        0x05,                           /* header name length */
+        'e',  'v',  'e',  'n',  't',    /* header name */
+        0x04,                           /* header value type */
+        0x00, 0x00,                     /* header value length */
+        0x00, 0x20,                     /* header value */
+        0x04, 0xa1, 0xd4, 0x7c          /* message crc */
+    };
+    /* clang-format on */
+
+    struct test_decoder_data decoder_data = {
+        .latest_payload = 0,
+        .written = 0,
+        .alloc = allocator,
+        .latest_error = 0,
+    };
+    aws_event_stream_headers_list_init(&decoder_data.headers_list, allocator);
+
+    struct aws_event_stream_streaming_decoder_options decoder_options = {
+        .on_payload_segment = s_decoder_test_on_payload_segment,
+        .on_prelude = s_decoder_test_on_prelude_received,
+        .on_header = s_decoder_test_header_received,
+        .on_complete = s_decoder_test_on_complete,
+        .on_error = s_decoder_test_on_error,
+        .user_data = &decoder_data};
+
+    struct aws_event_stream_streaming_decoder decoder;
+    aws_event_stream_streaming_decoder_init_from_options(&decoder, allocator, &decoder_options);
+
+    struct aws_byte_buf test_buf = aws_byte_buf_from_array(test_data, sizeof(test_data));
+
+    ASSERT_SUCCESS(
+        aws_event_stream_streaming_decoder_pump(&decoder, &test_buf), "Message validation should have succeeded");
+    ASSERT_SUCCESS(decoder_data.latest_error, "No Error callback shouldn't have been called");
+
+    ASSERT_INT_EQUALS(0x0000001B, decoder_data.latest_prelude.total_len, "Message length should have been 0x1B");
+    ASSERT_INT_EQUALS(0x0000000B, decoder_data.latest_prelude.headers_len, "Headers Length should have been 0xB");
+    ASSERT_INT_EQUALS(0xE5C0A072, decoder_data.latest_prelude.prelude_crc, "Prelude CRC should have been 0xE5C0A072");
+
+    const char *expected_header_name = "event";
+    struct aws_event_stream_header_value_pair latest_header;
+    aws_array_list_get_at(&decoder_data.headers_list, &latest_header, 0);
+
+    ASSERT_BIN_ARRAYS_EQUALS(
+        expected_header_name,
+        strlen(expected_header_name),
+        latest_header.header_name,
+        latest_header.header_name_len,
+        "header name should have been %s",
+        expected_header_name);
+
+    int32_t latest_header_value = aws_event_stream_header_value_as_int32(&latest_header);
+    ASSERT_INT_EQUALS(0x00000020, latest_header_value, "Header value should have been 0x00000020");
+    ASSERT_UINT_EQUALS(0x04A1D47C, decoder_data.message_crc);
+
+    aws_event_stream_headers_list_cleanup(&decoder_data.headers_list);
+    return 0;
+}
+
+AWS_TEST_CASE(
+    test_streaming_decoder_incoming_application_one_int32_header_pair_valid,
+    s_test_streaming_decoder_incoming_application_one_int32_header_pair_valid_fn)
+
+static int s_test_streaming_decoder_incoming_application_one_bool_header_pair_valid_fn(
+    struct aws_allocator *allocator,
+    void *ctx) {
+    (void)ctx;
+    /* clang-format off */
+    uint8_t test_data[] = {
+        0x00, 0x00, 0x00, 0x17,         /* total length */
+        0x00, 0x00, 0x00, 0x07,         /* headers length */
+        0x29, 0x86, 0x01, 0x58,         /* prelude crc */
+        0x05,                           /* header name length */
+        'e',  'v',  'e',  'n',  't',    /* header name */
+        0x00,                           /* header value type */
+        0x4b, 0x4d, 0x2b, 0xe7          /* message crc */
+    };
+    /* clang-format on */
+
+    struct test_decoder_data decoder_data = {
+        .latest_payload = 0,
+        .written = 0,
+        .alloc = allocator,
+        .latest_error = 0,
+    };
+    aws_event_stream_headers_list_init(&decoder_data.headers_list, allocator);
+
+    struct aws_event_stream_streaming_decoder_options decoder_options = {
+        .on_payload_segment = s_decoder_test_on_payload_segment,
+        .on_prelude = s_decoder_test_on_prelude_received,
+        .on_header = s_decoder_test_header_received,
+        .on_complete = s_decoder_test_on_complete,
+        .on_error = s_decoder_test_on_error,
+        .user_data = &decoder_data};
+
+    struct aws_event_stream_streaming_decoder decoder;
+    aws_event_stream_streaming_decoder_init_from_options(&decoder, allocator, &decoder_options);
+
+    struct aws_byte_buf test_buf = aws_byte_buf_from_array(test_data, sizeof(test_data));
+
+    ASSERT_SUCCESS(
+        aws_event_stream_streaming_decoder_pump(&decoder, &test_buf), "Message validation should have succeeded");
+    ASSERT_SUCCESS(decoder_data.latest_error, "No Error callback shouldn't have been called");
+
+    ASSERT_INT_EQUALS(0x00000017, decoder_data.latest_prelude.total_len, "Message length should have been 0x17");
+    ASSERT_INT_EQUALS(0x00000007, decoder_data.latest_prelude.headers_len, "Headers Length should have been 0x7");
+    ASSERT_INT_EQUALS(0x29860158, decoder_data.latest_prelude.prelude_crc, "Prelude CRC should have been 0x29860158");
+
+    const char *expected_header_name = "event";
+    struct aws_event_stream_header_value_pair latest_header;
+    aws_array_list_get_at(&decoder_data.headers_list, &latest_header, 0);
+
+    ASSERT_BIN_ARRAYS_EQUALS(
+        expected_header_name,
+        strlen(expected_header_name),
+        latest_header.header_name,
+        latest_header.header_name_len,
+        "header name should have been %s",
+        expected_header_name);
+
+    int8_t latest_header_value = aws_event_stream_header_value_as_bool(&latest_header);
+    ASSERT_INT_EQUALS(1, latest_header_value, "Header value should have been true");
+    ASSERT_UINT_EQUALS(0x4B4D2BE7, decoder_data.message_crc);
+
+    aws_event_stream_headers_list_cleanup(&decoder_data.headers_list);
+    return 0;
+}
+
+AWS_TEST_CASE(
+    test_streaming_decoder_incoming_application_one_bool_header_pair_valid,
+    s_test_streaming_decoder_incoming_application_one_bool_header_pair_valid_fn)
+
 static int s_test_streaming_decoder_incoming_multiple_messages_fn(struct aws_allocator *allocator, void *ctx) {
     (void)ctx;
     uint8_t test_data[] = {
@@ -362,16 +521,18 @@ static int s_test_streaming_decoder_incoming_multiple_messages_fn(struct aws_all
                                 This will fall into the middle of message boundaries and preludes. */
 
     struct test_decoder_data decoder_data = {.latest_payload = 0, .written = 0, .alloc = allocator, .latest_error = 0};
+    aws_event_stream_headers_list_init(&decoder_data.headers_list, allocator);
+
+    struct aws_event_stream_streaming_decoder_options decoder_options = {
+        .on_payload_segment = s_decoder_test_on_payload_segment,
+        .on_prelude = s_decoder_test_on_prelude_received,
+        .on_header = s_decoder_test_header_received,
+        .on_complete = s_decoder_test_on_complete,
+        .on_error = s_decoder_test_on_error,
+        .user_data = &decoder_data};
 
     struct aws_event_stream_streaming_decoder decoder;
-    aws_event_stream_streaming_decoder_init(
-        &decoder,
-        allocator,
-        s_decoder_test_on_payload_segment,
-        s_decoder_test_on_prelude_received,
-        s_decoder_test_header_received,
-        s_decoder_test_on_error,
-        &decoder_data);
+    aws_event_stream_streaming_decoder_init_from_options(&decoder, allocator, &decoder_options);
 
     size_t current_written = 0;
     int err_code = 0;
@@ -390,6 +551,7 @@ static int s_test_streaming_decoder_incoming_multiple_messages_fn(struct aws_all
     ASSERT_INT_EQUALS(0x00000000, decoder_data.latest_prelude.headers_len, "Headers Length should have been 0x00");
     ASSERT_INT_EQUALS(0x05c248eb, decoder_data.latest_prelude.prelude_crc, "Prelude CRC should have been 0x8c335472");
     ASSERT_INT_EQUALS(0, decoder_data.written, "No payload data should have been written");
+    ASSERT_UINT_EQUALS(0x7D98C8FF, decoder_data.message_crc);
 
     while (current_written < sizeof(test_data) && !err_code) {
         size_t to_write =
@@ -410,18 +572,22 @@ static int s_test_streaming_decoder_incoming_multiple_messages_fn(struct aws_all
     const char *content_type = "content-type";
     const char *content_type_value = "application/json";
 
+    struct aws_event_stream_header_value_pair latest_header;
+    aws_array_list_get_at(&decoder_data.headers_list, &latest_header, 0);
+    struct aws_byte_buf latest_header_value = aws_event_stream_header_value_as_string(&latest_header);
+
     ASSERT_BIN_ARRAYS_EQUALS(
         content_type,
         strlen(content_type),
-        decoder_data.latest_header_name,
-        strlen(decoder_data.latest_header_name),
+        latest_header.header_name,
+        latest_header.header_name_len,
         "header name should have been %s",
         content_type);
     ASSERT_BIN_ARRAYS_EQUALS(
         content_type_value,
         strlen(content_type_value),
-        decoder_data.latest_header_value,
-        strlen(decoder_data.latest_header_value),
+        latest_header_value.buffer,
+        latest_header_value.len,
         "header value should have been %s",
         content_type_value);
 
@@ -442,8 +608,10 @@ static int s_test_streaming_decoder_incoming_multiple_messages_fn(struct aws_all
     if (decoder_data.latest_payload) {
         aws_mem_release(allocator, decoder_data.latest_payload);
     }
+    ASSERT_UINT_EQUALS(0x8D9C08B1, decoder_data.message_crc);
 
     aws_event_stream_streaming_decoder_clean_up(&decoder);
+    aws_event_stream_headers_list_cleanup(&decoder_data.headers_list);
 
     return 0;
 }
