@@ -442,7 +442,7 @@ static struct aws_http_message *s_build_http_request(
         if (data_len > 0) {
             char content_length[64];
             AWS_ZERO_ARRAY(content_length);
-            sprintf(content_length, "%" PRIi64, data_len);
+            snprintf(content_length, sizeof(content_length), "%" PRIi64, data_len);
             struct aws_http_header content_length_header = {
                 .name = aws_byte_cursor_from_c_str("content-length"),
                 .value = aws_byte_cursor_from_c_str(content_length),
@@ -514,15 +514,6 @@ static void s_on_signing_complete(struct aws_http_message *request, int error_co
         exit(1);
     }
 
-    size_t final_header_count = aws_http_message_get_header_count(app_ctx->request);
-
-    struct aws_http_header headers[20];
-    AWS_ASSERT(final_header_count <= AWS_ARRAY_SIZE(headers));
-    AWS_ZERO_ARRAY(headers);
-    for (size_t i = 0; i < final_header_count; ++i) {
-        aws_http_message_get_header(app_ctx->request, &headers[i], i);
-    }
-
     struct aws_http_make_request_options final_request = {
         .self_size = sizeof(final_request),
         .user_data = app_ctx,
