@@ -10,6 +10,8 @@
 #include <aws/io/logging.h>
 #include <aws/s3/exports.h>
 
+AWS_PUSH_SANE_WARNING_LEVEL
+
 #define AWS_C_S3_PACKAGE_ID 14
 
 enum aws_s3_errors {
@@ -33,6 +35,12 @@ enum aws_s3_errors {
     AWS_ERROR_S3_LIST_PARTS_PARSE_FAILED,
     AWS_ERROR_S3_RESUMED_PART_CHECKSUM_MISMATCH,
     AWS_ERROR_S3_RESUME_FAILED,
+    AWS_ERROR_S3_OBJECT_MODIFIED,
+    AWS_ERROR_S3_NON_RECOVERABLE_ASYNC_ERROR,
+    AWS_ERROR_S3_METRIC_DATA_NOT_AVAILABLE,
+    AWS_ERROR_S3_INCORRECT_CONTENT_LENGTH,
+    AWS_ERROR_S3_REQUEST_TIME_TOO_SKEWED,
+    AWS_ERROR_S3_FILE_MODIFIED,
     AWS_ERROR_S3_END_RANGE = AWS_ERROR_ENUM_END_RANGE(AWS_C_S3_PACKAGE_ID)
 };
 
@@ -55,6 +63,12 @@ struct aws_s3_cpu_group_info {
     size_t nic_name_array_length;
 };
 
+#ifdef _MSC_VER
+#    pragma warning(push)
+#    pragma warning(disable : 4626) /* assignment operator was implicitly defined as deleted */
+#    pragma warning(disable : 5027) /* move assignment operator was implicitly defined as deleted */
+#endif
+
 struct aws_s3_compute_platform_info {
     /* name of the instance-type: example c5n.18xlarge */
     const struct aws_byte_cursor instance_type;
@@ -66,6 +80,10 @@ struct aws_s3_compute_platform_info {
     size_t cpu_group_info_array_length;
 };
 
+#ifdef _MSC_VER
+#    pragma warning(pop)
+#endif
+
 AWS_EXTERN_C_BEGIN
 
 /**
@@ -89,5 +107,6 @@ AWS_S3_API
 void aws_s3_library_clean_up(void);
 
 AWS_EXTERN_C_END
+AWS_POP_SANE_WARNING_LEVEL
 
 #endif /* AWS_S3_H */
