@@ -7,6 +7,7 @@
 #include "aws/s3/private/s3_checksums.h"
 #include "aws/s3/private/s3_client_impl.h"
 #include "aws/s3/private/s3_meta_request_impl.h"
+#include "aws/s3/private/s3_parallel_input_stream.h"
 #include "aws/s3/private/s3_request_messages.h"
 #include "aws/s3/private/s3_util.h"
 #include <aws/auth/signable.h>
@@ -25,6 +26,7 @@
 
 static const size_t s_dynamic_body_initial_buf_size = KB_TO_BYTES(1);
 static const size_t s_default_body_streaming_priority_queue_size = 16;
+static const size_t s_default_event_delivery_array_size = 16;
 
 static int s_s3_request_priority_queue_pred(const void *a, const void *b);
 static void s_s3_meta_request_destroy(void *user_data);
@@ -223,6 +225,18 @@ int aws_s3_meta_request_init_base(
         goto error;
     }
 
+    aws_array_list_init_dynamic(
+        &meta_request->synced_data.event_delivery_array,
+        meta_request->allocator,
+        s_default_event_delivery_array_size,
+        sizeof(struct aws_s3_meta_request_event));
+
+    aws_array_list_init_dynamic(
+        &meta_request->io_threaded_data.event_delivery_array,
+        meta_request->allocator,
+        s_default_event_delivery_array_size,
+        sizeof(struct aws_s3_meta_request_event));
+
     *((size_t *)&meta_request->part_size) = part_size;
     *((bool *)&meta_request->should_compute_content_md5) = should_compute_content_md5;
     checksum_config_init(&meta_request->checksum_config, options->checksum_config);
@@ -230,16 +244,25 @@ int aws_s3_meta_request_init_base(
         meta_request->cached_signing_config = aws_cached_signing_config_new(allocator, options->signing_config);
     }
 
+    /* Client is currently optional to allow spinning up a meta_request without a client in a test. */
+    if (client != NULL) {
+        meta_request->client = aws_s3_client_acquire(client);
+        meta_request->io_event_loop = aws_event_loop_group_get_next_loop(client->body_streaming_elg);
+        meta_request->synced_data.read_window_running_total = client->initial_read_window;
+    }
+
     /* Set initial_meta_request, based on how the request's body is being passed in
      * (we checked earlier that it's not being passed multiple ways) */
     if (options->send_filepath.len > 0) {
-        /* Create copy of original message, but with body-stream that reads directly from file */
-        meta_request->initial_request_message = aws_s3_message_util_copy_http_message_filepath_body_all_headers(
-            allocator, options->message, options->send_filepath);
-        if (meta_request->initial_request_message == NULL) {
+        /* Create parallel read stream from file */
+        meta_request->request_body_parallel_stream =
+            client->vtable->parallel_input_stream_new_from_file(allocator, options->send_filepath);
+        if (meta_request->request_body_parallel_stream == NULL) {
             goto error;
         }
 
+        /* but keep original message around for headers, method, etc */
+        meta_request->initial_request_message = aws_http_message_acquire(options->message);
     } else if (options->send_async_stream != NULL) {
         /* Read from async body-stream, but keep original message around for headers, method, etc */
         meta_request->request_body_async_stream = aws_async_input_stream_acquire(options->send_async_stream);
@@ -250,14 +273,6 @@ int aws_s3_meta_request_init_base(
         meta_request->initial_request_message = aws_http_message_acquire(options->message);
     }
 
-    /* Client is currently optional to allow spinning up a meta_request without a client in a test. */
-    if (client != NULL) {
-        aws_s3_client_acquire(client);
-        meta_request->client = client;
-        meta_request->io_event_loop = aws_event_loop_group_get_next_loop(client->body_streaming_elg);
-        meta_request->synced_data.read_window_running_total = client->initial_read_window;
-    }
-
     meta_request->synced_data.next_streaming_part = 1;
 
     meta_request->meta_request_level_running_response_sum = NULL;
@@ -353,6 +368,16 @@ void aws_s3_meta_request_set_fail_synced(
     AWS_PRECONDITION(meta_request);
     ASSERT_SYNCED_DATA_LOCK_HELD(meta_request);
 
+    /* Protect against bugs */
+    if (error_code == AWS_ERROR_SUCCESS) {
+        AWS_ASSERT(false);
+        AWS_LOGF_ERROR(
+            AWS_LS_S3_META_REQUEST,
+            "id=%p Meta request failed but error code not set, AWS_ERROR_UNKNOWN will be reported",
+            (void *)meta_request);
+        error_code = AWS_ERROR_UNKNOWN;
+    }
+
     if (meta_request->synced_data.finish_result_set) {
         return;
     }
@@ -447,7 +472,15 @@ static void s_s3_meta_request_destroy(void *user_data) {
     aws_s3_endpoint_release(meta_request->endpoint);
     meta_request->client = aws_s3_client_release(meta_request->client);
 
+    AWS_ASSERT(aws_priority_queue_size(&meta_request->synced_data.pending_body_streaming_requests) == 0);
     aws_priority_queue_clean_up(&meta_request->synced_data.pending_body_streaming_requests);
+
+    AWS_ASSERT(aws_array_list_length(&meta_request->synced_data.event_delivery_array) == 0);
+    aws_array_list_clean_up(&meta_request->synced_data.event_delivery_array);
+
+    AWS_ASSERT(aws_array_list_length(&meta_request->io_threaded_data.event_delivery_array) == 0);
+    aws_array_list_clean_up(&meta_request->io_threaded_data.event_delivery_array);
+
     aws_s3_meta_request_result_clean_up(meta_request, &meta_request->synced_data.finish_result);
 
     if (meta_request->vtable != NULL) {
@@ -597,7 +630,14 @@ static void s_s3_meta_request_schedule_prepare_request_default(
 
     aws_task_init(
         &payload->task, s_s3_meta_request_prepare_request_task, payload, "s3_meta_request_prepare_request_task");
-    aws_event_loop_schedule_task_now(meta_request->io_event_loop, &payload->task);
+    if (meta_request->request_body_parallel_stream) {
+        /* The body stream supports reading in parallel, so schedule task on any I/O thread.
+         * If we always used the meta-request's dedicated io_event_loop, we wouldn't get any parallelism. */
+        struct aws_event_loop *loop = aws_event_loop_group_get_next_loop(client->body_streaming_elg);
+        aws_event_loop_schedule_task_now(loop, &payload->task);
+    } else {
+        aws_event_loop_schedule_task_now(meta_request->io_event_loop, &payload->task);
+    }
 }
 
 static void s_s3_meta_request_prepare_request_task(struct aws_task *task, void *arg, enum aws_task_status task_status) {
@@ -951,12 +991,12 @@ static void s_get_response_part_finish_checksum_helper(struct aws_s3_connection
         request->validation_algorithm = request->request_level_running_response_sum->algorithm;
         aws_byte_buf_clean_up(&response_body_sum);
         aws_byte_buf_clean_up(&encoded_response_body_sum);
-        aws_checksum_destroy(request->request_level_running_response_sum);
-        aws_byte_buf_clean_up(&request->request_level_response_header_checksum);
-        request->request_level_running_response_sum = NULL;
     } else {
         request->did_validate = false;
     }
+    aws_checksum_destroy(request->request_level_running_response_sum);
+    aws_byte_buf_clean_up(&request->request_level_response_header_checksum);
+    request->request_level_running_response_sum = NULL;
 }
 
 static int s_s3_meta_request_incoming_headers(
@@ -1152,40 +1192,66 @@ static void s_s3_meta_request_send_request_finish(
     vtable->send_request_finish(connection, stream, error_code);
 }
 
-static int s_s3_meta_request_error_code_from_response_body(struct aws_s3_request *request) {
-    AWS_PRECONDITION(request);
-    if (request->send_data.response_body.len == 0) {
-        /* Empty body is success */
-        return AWS_ERROR_SUCCESS;
+/* Return whether the response to this request might contain an error, even though we got 200 OK.
+ * see: https://repost.aws/knowledge-center/s3-resolve-200-internalerror */
+static bool s_should_check_for_error_despite_200_OK(const struct aws_s3_request *request) {
+    /* We handle async error for every request BUT get object. */
+    struct aws_s3_meta_request *meta_request = request->meta_request;
+    if (meta_request->type == AWS_S3_META_REQUEST_TYPE_GET_OBJECT) {
+        return false;
     }
-    struct aws_byte_cursor response_body_cursor = aws_byte_cursor_from_buf(&request->send_data.response_body);
-    struct aws_byte_cursor error_code_string = {0};
-    const char *xml_path[] = {"Error", "Code", NULL};
-    if (aws_xml_get_body_at_path(request->allocator, response_body_cursor, xml_path, &error_code_string)) {
+    return true;
+}
 
-        if (aws_last_error() == AWS_ERROR_INVALID_XML || aws_last_error() == AWS_ERROR_STRING_MATCH_NOT_FOUND) {
-            /* The xml body is not Error, we can safely think the request succeed. */
-            aws_reset_error();
-            return AWS_ERROR_SUCCESS;
-        } else {
-            return aws_last_error();
+static int s_s3_meta_request_error_code_from_response(struct aws_s3_request *request) {
+    AWS_PRECONDITION(request);
+
+    int error_code_from_status = s_s3_meta_request_error_code_from_response_status(request->send_data.response_status);
+
+    /* Response body might be XML with an <Error><Code> inside.
+     * The is very likely when status-code is bad.
+     * In some cases, it's even possible after 200 OK. */
+    int error_code_from_xml = AWS_ERROR_SUCCESS;
+    if (error_code_from_status != AWS_ERROR_SUCCESS || s_should_check_for_error_despite_200_OK(request)) {
+        if (request->send_data.response_body.len > 0) {
+            /* Attempt to read as XML, it's fine if this fails. */
+            struct aws_byte_cursor xml_doc = aws_byte_cursor_from_buf(&request->send_data.response_body);
+            struct aws_byte_cursor error_code_string = {0};
+            const char *xml_path[] = {"Error", "Code", NULL};
+            if (aws_xml_get_body_at_path(request->allocator, xml_doc, xml_path, &error_code_string) == AWS_OP_SUCCESS) {
+                /* Found an <Error><Code> string! Map it to CRT error code. */
+                error_code_from_xml = aws_s3_crt_error_code_from_server_error_code_string(error_code_string);
+            }
+        }
+    }
+
+    if (error_code_from_status == AWS_ERROR_SUCCESS) {
+        /* Status-code was OK, so assume everything's good, unless we found an <Error><Code> in the XML */
+        switch (error_code_from_xml) {
+            case AWS_ERROR_SUCCESS:
+                return AWS_ERROR_SUCCESS;
+            case AWS_ERROR_UNKNOWN:
+                return AWS_ERROR_S3_NON_RECOVERABLE_ASYNC_ERROR;
+            default:
+                return error_code_from_xml;
         }
     } else {
-        /* Check the error code. Map the S3 error code to CRT error code. */
-        int error_code = aws_s3_crt_error_code_from_server_error_code_string(error_code_string);
-        if (error_code == AWS_ERROR_UNKNOWN) {
-            /* All error besides of internal error from async error are not recoverable from retry for now. */
-            error_code = AWS_ERROR_S3_NON_RECOVERABLE_ASYNC_ERROR;
+        /* Return error based on status-code, unless we got something more specific from XML */
+        switch (error_code_from_xml) {
+            case AWS_ERROR_SUCCESS:
+                return error_code_from_status;
+            case AWS_ERROR_UNKNOWN:
+                return error_code_from_status;
+            default:
+                return error_code_from_xml;
         }
-        return error_code;
     }
 }
 
-static void s_s3_meta_request_send_request_finish_helper(
+void aws_s3_meta_request_send_request_finish_default(
     struct aws_s3_connection *connection,
     struct aws_http_stream *stream,
-    int error_code,
-    bool handle_async_error) {
+    int error_code) {
 
     struct aws_s3_request *request = connection->request;
     AWS_PRECONDITION(request);
@@ -1200,12 +1266,7 @@ static void s_s3_meta_request_send_request_finish_helper(
     /* If our error code is currently success, then we have some other calls to make that could still indicate a
      * failure. */
     if (error_code == AWS_ERROR_SUCCESS) {
-        if (handle_async_error && response_status == AWS_HTTP_STATUS_CODE_200_OK) {
-            error_code = s_s3_meta_request_error_code_from_response_body(request);
-        } else {
-            error_code = s_s3_meta_request_error_code_from_response_status(response_status);
-        }
-
+        error_code = s_s3_meta_request_error_code_from_response(request);
         if (error_code != AWS_ERROR_SUCCESS) {
             aws_raise_error(error_code);
         }
@@ -1284,20 +1345,6 @@ static void s_s3_meta_request_send_request_finish_helper(
     aws_s3_client_notify_connection_finished(client, connection, error_code, finish_code);
 }
 
-void aws_s3_meta_request_send_request_finish_default(
-    struct aws_s3_connection *connection,
-    struct aws_http_stream *stream,
-    int error_code) {
-    s_s3_meta_request_send_request_finish_helper(connection, stream, error_code, false /*async error*/);
-}
-
-void aws_s3_meta_request_send_request_finish_handle_async_error(
-    struct aws_s3_connection *connection,
-    struct aws_http_stream *stream,
-    int error_code) {
-    s_s3_meta_request_send_request_finish_helper(connection, stream, error_code, true /*async error*/);
-}
-
 void aws_s3_meta_request_finished_request(
     struct aws_s3_meta_request *meta_request,
     struct aws_s3_request *request,
@@ -1309,12 +1356,6 @@ void aws_s3_meta_request_finished_request(
     meta_request->vtable->finished_request(meta_request, request, error_code);
 }
 
-struct s3_stream_response_body_payload {
-    struct aws_s3_meta_request *meta_request;
-    struct aws_linked_list requests;
-    struct aws_task task;
-};
-
 /* Pushes a request into the body streaming priority queue. Derived meta request types should not call this--they
  * should instead call aws_s3_meta_request_stream_response_body_synced.*/
 static void s_s3_meta_request_body_streaming_push_synced(
@@ -1327,19 +1368,17 @@ static void s_s3_meta_request_body_streaming_push_synced(
 static struct aws_s3_request *s_s3_meta_request_body_streaming_pop_next_synced(
     struct aws_s3_meta_request *meta_request);
 
-static void s_s3_meta_request_body_streaming_task(struct aws_task *task, void *arg, enum aws_task_status task_status);
+static void s_s3_meta_request_event_delivery_task(struct aws_task *task, void *arg, enum aws_task_status task_status);
 
 void aws_s3_meta_request_stream_response_body_synced(
     struct aws_s3_meta_request *meta_request,
     struct aws_s3_request *request) {
+
     ASSERT_SYNCED_DATA_LOCK_HELD(meta_request);
     AWS_PRECONDITION(meta_request);
     AWS_PRECONDITION(request);
     AWS_PRECONDITION(request->part_number > 0);
 
-    struct aws_linked_list streaming_requests;
-    aws_linked_list_init(&streaming_requests);
-
     /* Push it into the priority queue. */
     s_s3_meta_request_body_streaming_push_synced(meta_request, request);
 
@@ -1347,49 +1386,62 @@ void aws_s3_meta_request_stream_response_body_synced(
     AWS_PRECONDITION(client);
     aws_atomic_fetch_add(&client->stats.num_requests_stream_queued_waiting, 1);
 
-    /* Grab the next request that can be streamed back to the caller. */
-    struct aws_s3_request *next_streaming_request = s_s3_meta_request_body_streaming_pop_next_synced(meta_request);
+    /* Grab any requests that can be streamed back to the caller
+     * and send them for delivery on io_event_loop thread. */
     uint32_t num_streaming_requests = 0;
+    struct aws_s3_request *next_streaming_request;
+    while ((next_streaming_request = s_s3_meta_request_body_streaming_pop_next_synced(meta_request)) != NULL) {
+        struct aws_s3_meta_request_event event = {.type = AWS_S3_META_REQUEST_EVENT_RESPONSE_BODY};
+        event.u.response_body.completed_request = next_streaming_request;
+        aws_s3_meta_request_add_event_for_delivery_synced(meta_request, &event);
 
-    /* Grab any additional requests that could be streamed to the caller. */
-    while (next_streaming_request != NULL) {
-        aws_atomic_fetch_sub(&client->stats.num_requests_stream_queued_waiting, 1);
-
-        aws_linked_list_push_back(&streaming_requests, &next_streaming_request->node);
         ++num_streaming_requests;
-        next_streaming_request = s_s3_meta_request_body_streaming_pop_next_synced(meta_request);
     }
 
-    if (aws_linked_list_empty(&streaming_requests)) {
+    if (num_streaming_requests == 0) {
         return;
     }
 
-    aws_atomic_fetch_add(&client->stats.num_requests_streaming, num_streaming_requests);
+    aws_atomic_fetch_add(&client->stats.num_requests_streaming_response, num_streaming_requests);
+    aws_atomic_fetch_sub(&client->stats.num_requests_stream_queued_waiting, num_streaming_requests);
 
     meta_request->synced_data.num_parts_delivery_sent += num_streaming_requests;
-
-    struct s3_stream_response_body_payload *payload =
-        aws_mem_calloc(client->allocator, 1, sizeof(struct s3_stream_response_body_payload));
-
-    aws_s3_meta_request_acquire(meta_request);
-    payload->meta_request = meta_request;
-
-    aws_linked_list_init(&payload->requests);
-    aws_linked_list_swap_contents(&payload->requests, &streaming_requests);
-
-    aws_task_init(
-        &payload->task, s_s3_meta_request_body_streaming_task, payload, "s_s3_meta_request_body_streaming_task");
-    aws_event_loop_schedule_task_now(meta_request->io_event_loop, &payload->task);
 }
 
-static void s_s3_meta_request_body_streaming_task(struct aws_task *task, void *arg, enum aws_task_status task_status) {
+void aws_s3_meta_request_add_event_for_delivery_synced(
+    struct aws_s3_meta_request *meta_request,
+    const struct aws_s3_meta_request_event *event) {
+
+    ASSERT_SYNCED_DATA_LOCK_HELD(meta_request);
+
+    aws_array_list_push_back(&meta_request->synced_data.event_delivery_array, event);
+
+    /* If the array was empty before, schedule task to deliver all events in the array.
+     * If the array already had things in it, then the task is already scheduled and will run soon. */
+    if (aws_array_list_length(&meta_request->synced_data.event_delivery_array) == 1) {
+        aws_s3_meta_request_acquire(meta_request);
+
+        aws_task_init(
+            &meta_request->synced_data.event_delivery_task,
+            s_s3_meta_request_event_delivery_task,
+            meta_request,
+            "s3_meta_request_event_delivery");
+        aws_event_loop_schedule_task_now(meta_request->io_event_loop, &meta_request->synced_data.event_delivery_task);
+    }
+}
+
+bool aws_s3_meta_request_are_events_out_for_delivery_synced(struct aws_s3_meta_request *meta_request) {
+    ASSERT_SYNCED_DATA_LOCK_HELD(meta_request);
+    return aws_array_list_length(&meta_request->synced_data.event_delivery_array) > 0 ||
+           meta_request->synced_data.event_delivery_active;
+}
+
+/* Deliver events in event_delivery_array.
+ * This task runs on the meta-request's io_event_loop thread. */
+static void s_s3_meta_request_event_delivery_task(struct aws_task *task, void *arg, enum aws_task_status task_status) {
     (void)task;
     (void)task_status;
-
-    struct s3_stream_response_body_payload *payload = arg;
-    AWS_PRECONDITION(payload);
-
-    struct aws_s3_meta_request *meta_request = payload->meta_request;
+    struct aws_s3_meta_request *meta_request = arg;
     AWS_PRECONDITION(meta_request);
     AWS_PRECONDITION(meta_request->vtable);
 
@@ -1399,40 +1451,99 @@ static void s_s3_meta_request_body_streaming_task(struct aws_task *task, void *a
     /* Client owns this event loop group. A cancel should not be possible. */
     AWS_ASSERT(task_status == AWS_TASK_STATUS_RUN_READY);
 
-    struct aws_linked_list completed_requests;
-    aws_linked_list_init(&completed_requests);
+    /* Swap contents of synced_data.event_delivery_array into this pre-allocated array-list, then process events */
+    struct aws_array_list *event_delivery_array = &meta_request->io_threaded_data.event_delivery_array;
+    AWS_FATAL_ASSERT(aws_array_list_length(event_delivery_array) == 0);
 
+    /* If an error occurs, don't fire callbacks anymore. */
     int error_code = AWS_ERROR_SUCCESS;
-    uint32_t num_successful = 0;
-    uint32_t num_failed = 0;
+    uint32_t num_parts_delivered = 0;
 
-    while (!aws_linked_list_empty(&payload->requests)) {
-        struct aws_linked_list_node *request_node = aws_linked_list_pop_front(&payload->requests);
-        struct aws_s3_request *request = AWS_CONTAINER_OF(request_node, struct aws_s3_request, node);
-        AWS_ASSERT(meta_request == request->meta_request);
-        struct aws_byte_cursor body_buffer_byte_cursor = aws_byte_cursor_from_buf(&request->send_data.response_body);
+    /* BEGIN CRITICAL SECTION */
+    {
+        aws_s3_meta_request_lock_synced_data(meta_request);
 
-        AWS_ASSERT(request->part_number >= 1);
+        aws_array_list_swap_contents(event_delivery_array, &meta_request->synced_data.event_delivery_array);
+        meta_request->synced_data.event_delivery_active = true;
 
-        if (aws_s3_meta_request_has_finish_result(meta_request)) {
-            ++num_failed;
-        } else {
-            if (body_buffer_byte_cursor.len > 0 && error_code == AWS_ERROR_SUCCESS && meta_request->body_callback &&
-                meta_request->body_callback(
-                    meta_request, &body_buffer_byte_cursor, request->part_range_start, meta_request->user_data)) {
-                error_code = aws_last_error_or_unknown();
-            }
-
-            if (error_code == AWS_ERROR_SUCCESS) {
-                ++num_successful;
-            } else {
-                ++num_failed;
-            }
+        if (aws_s3_meta_request_has_finish_result_synced(meta_request)) {
+            error_code = AWS_ERROR_S3_CANCELED;
         }
 
-        aws_atomic_fetch_sub(&client->stats.num_requests_streaming, 1);
-        aws_s3_request_release(request);
+        aws_s3_meta_request_unlock_synced_data(meta_request);
     }
+    /* END CRITICAL SECTION */
+
+    /* Deliver all events */
+    for (size_t event_i = 0; event_i < aws_array_list_length(event_delivery_array); ++event_i) {
+        struct aws_s3_meta_request_event event;
+        aws_array_list_get_at(event_delivery_array, &event, event_i);
+        switch (event.type) {
+
+            case AWS_S3_META_REQUEST_EVENT_RESPONSE_BODY: {
+                struct aws_s3_request *request = event.u.response_body.completed_request;
+                AWS_ASSERT(meta_request == request->meta_request);
+                struct aws_byte_cursor response_body = aws_byte_cursor_from_buf(&request->send_data.response_body);
+
+                AWS_ASSERT(request->part_number >= 1);
+
+                if (error_code == AWS_ERROR_SUCCESS && response_body.len > 0 && meta_request->body_callback != NULL) {
+                    if (meta_request->body_callback(
+                            meta_request, &response_body, request->part_range_start, meta_request->user_data)) {
+
+                        error_code = aws_last_error_or_unknown();
+                        AWS_LOGF_ERROR(
+                            AWS_LS_S3_META_REQUEST,
+                            "id=%p Response body callback raised error %d (%s).",
+                            (void *)meta_request,
+                            error_code,
+                            aws_error_str(error_code));
+                    }
+                }
+                aws_atomic_fetch_sub(&client->stats.num_requests_streaming_response, 1);
+
+                ++num_parts_delivered;
+                aws_s3_request_release(request);
+            } break;
+
+            case AWS_S3_META_REQUEST_EVENT_PROGRESS: {
+                if (error_code == AWS_ERROR_SUCCESS && meta_request->progress_callback != NULL) {
+                    /* Don't report 0 byte progress events.
+                     * The reasoning behind this is:
+                     *
+                     * In some code paths, when no data is transferred, there are no progress events,
+                     * but in other code paths there might be one progress event of 0 bytes.
+                     * We want to be consistent, either:
+                     * - REPORT AT LEAST ONCE: even if no data is being transferred.
+                     *   This would require finding every code path where no progress events are sent,
+                     *   and sending an appropriate progress event, even if it's for 0 bytes.
+                     *   One example of ending early is: when resuming a paused upload,
+                     *   we do ListParts on the UploadID, and if that 404s we assume the
+                     *   previous "paused" meta-request actually completed,
+                     *   and so we immediately end the "resuming" meta-request
+                     *   as successful without sending any further HTTP requests.
+                     *   It would be tough to accurately report progress here because
+                     *   we don't know the total size, since we never read the request body,
+                     *   and didn't get any info about the previous upload.
+                     * OR
+                     * - NEVER REPORT ZERO BYTES: even if that means no progress events at all.
+                     *   This is easy to do. We'd only send progress events when data is transferred,
+                     *   and if a 0 byte event slips through somehow, just check before firing the callback.
+                     * Since the NEVER REPORT ZERO BYTES path is simpler to implement, we went with that. */
+                    if (event.u.progress.info.bytes_transferred > 0) {
+                        meta_request->progress_callback(meta_request, &event.u.progress.info, meta_request->user_data);
+                    }
+                }
+            } break;
+
+            default:
+                AWS_FATAL_ASSERT(false);
+        }
+    }
+
+    /* Done delivering events */
+    aws_array_list_clear(event_delivery_array);
+
     /* BEGIN CRITICAL SECTION */
     {
         aws_s3_meta_request_lock_synced_data(meta_request);
@@ -1440,14 +1551,11 @@ static void s_s3_meta_request_body_streaming_task(struct aws_task *task, void *a
             aws_s3_meta_request_set_fail_synced(meta_request, NULL, error_code);
         }
 
-        meta_request->synced_data.num_parts_delivery_completed += (num_failed + num_successful);
-        meta_request->synced_data.num_parts_delivery_failed += num_failed;
-        meta_request->synced_data.num_parts_delivery_succeeded += num_successful;
+        meta_request->synced_data.num_parts_delivery_completed += num_parts_delivered;
+        meta_request->synced_data.event_delivery_active = false;
         aws_s3_meta_request_unlock_synced_data(meta_request);
     }
     /* END CRITICAL SECTION */
-    aws_mem_release(client->allocator, payload);
-    payload = NULL;
 
     aws_s3_client_schedule_process_work(client);
     aws_s3_meta_request_release(meta_request);
@@ -1575,6 +1683,8 @@ void aws_s3_meta_request_finish_default(struct aws_s3_meta_request *meta_request
      * that, the downstream high level language doesn't need to wait for shutdown to clean related resource (eg: input
      * stream) */
     meta_request->request_body_async_stream = aws_async_input_stream_release(meta_request->request_body_async_stream);
+    meta_request->request_body_parallel_stream =
+        aws_parallel_input_stream_release(meta_request->request_body_parallel_stream);
     meta_request->initial_request_message = aws_http_message_release(meta_request->initial_request_message);
 
     if (meta_request->finish_callback != NULL) {
@@ -1591,6 +1701,7 @@ void aws_s3_meta_request_finish_default(struct aws_s3_meta_request *meta_request
 
 struct aws_future_bool *aws_s3_meta_request_read_body(
     struct aws_s3_meta_request *meta_request,
+    uint64_t offset,
     struct aws_byte_buf *buffer) {
 
     AWS_PRECONDITION(meta_request);
@@ -1601,6 +1712,11 @@ struct aws_future_bool *aws_s3_meta_request_read_body(
         return aws_async_input_stream_read_to_fill(meta_request->request_body_async_stream, buffer);
     }
 
+    /* If parallel-stream, simply call read(), which must fill the buffer and/or EOF */
+    if (meta_request->request_body_parallel_stream != NULL) {
+        return aws_parallel_input_stream_read(meta_request->request_body_parallel_stream, offset, buffer);
+    }
+
     /* Else synchronous aws_input_stream */
     struct aws_input_stream *synchronous_stream =
         aws_http_message_get_body_stream(meta_request->initial_request_message);
