@@ -61,11 +61,6 @@ const struct aws_byte_cursor g_user_agent_header_name = AWS_BYTE_CUR_INIT_FROM_S
 const struct aws_byte_cursor g_user_agent_header_product_name =
     AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("CRTS3NativeClient");
 
-const struct aws_byte_cursor g_s3_internal_error_code = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("InternalError");
-const struct aws_byte_cursor g_s3_slow_down_error_code = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("SlowDown");
-/* The special error code as Asynchronous Error Codes */
-const struct aws_byte_cursor g_s3_internal_errors_code = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("InternalErrors");
-
 const uint32_t g_s3_max_num_upload_parts = 10000;
 const size_t g_s3_min_upload_part_size = MB_TO_BYTES(5);
 
@@ -210,7 +205,7 @@ struct aws_cached_signing_config_aws *aws_cached_signing_config_new(
 
     AWS_ASSERT(aws_byte_cursor_is_valid(&signing_config->signed_body_value));
 
-    if (signing_config->service.len > 0) {
+    if (signing_config->signed_body_value.len > 0) {
         cached_signing_config->signed_body_value =
             aws_string_new_from_cursor(allocator, &signing_config->signed_body_value);
 
@@ -470,7 +465,7 @@ uint32_t aws_s3_get_num_parts(size_t part_size, uint64_t object_range_start, uin
 
     /* If the range has room for a second part, calculate the additional amount of parts. */
     if (second_part_start <= object_range_end) {
-        uint64_t aligned_range_remainder = object_range_end + 1 - second_part_start;
+        uint64_t aligned_range_remainder = object_range_end + 1 - second_part_start; /* range-end is inclusive */
         num_parts += (uint32_t)(aligned_range_remainder / (uint64_t)part_size);
 
         if ((aligned_range_remainder % part_size) > 0) {
@@ -515,7 +510,7 @@ void aws_s3_get_part_range(
         /* Else, find the next part by adding the object range + total number of whole parts before this one + initial
          * part size*/
         *out_part_range_start = object_range_start + ((uint64_t)(part_index - 1)) * part_size_uint64 + first_part_size;
-        *out_part_range_end = *out_part_range_start + part_size_uint64 - 1;
+        *out_part_range_end = *out_part_range_start + part_size_uint64 - 1; /* range-end is inclusive */
     }
 
     /* Cap the part's range end using the object's range end. */
@@ -583,12 +578,15 @@ int aws_s3_calculate_optimal_mpu_part_size_and_num_parts(
 }
 
 int aws_s3_crt_error_code_from_server_error_code_string(struct aws_byte_cursor error_code_string) {
-    if (aws_byte_cursor_eq_ignore_case(&error_code_string, &g_s3_slow_down_error_code)) {
+    if (aws_byte_cursor_eq_c_str_ignore_case(&error_code_string, "SlowDown")) {
         return AWS_ERROR_S3_SLOW_DOWN;
     }
-    if (aws_byte_cursor_eq_ignore_case(&error_code_string, &g_s3_internal_error_code) ||
-        aws_byte_cursor_eq_ignore_case(&error_code_string, &g_s3_internal_errors_code)) {
+    if (aws_byte_cursor_eq_c_str_ignore_case(&error_code_string, "InternalError") ||
+        aws_byte_cursor_eq_c_str_ignore_case(&error_code_string, "InternalErrors")) {
         return AWS_ERROR_S3_INTERNAL_ERROR;
     }
+    if (aws_byte_cursor_eq_c_str_ignore_case(&error_code_string, "RequestTimeTooSkewed")) {
+        return AWS_ERROR_S3_REQUEST_TIME_TOO_SKEWED;
+    }
     return AWS_ERROR_UNKNOWN;
 }
