@@ -91,3 +91,10 @@ void EVP_MD_do_all_sorted(void (*callback)(const EVP_MD *cipher,
   callback(EVP_sha512(), "sha512", NULL, arg);
   callback(EVP_sha512_256(), "sha512-256", NULL, arg);
 }
+
+void EVP_MD_do_all(void (*callback)(const EVP_MD *cipher,
+                                    const char *name, const char *unused,
+                                    void *arg),
+                          void *arg) {
+    EVP_MD_do_all_sorted(callback, arg);
+}
