@@ -33,10 +33,12 @@
 // when Fiat-crypto is used, or as:
 //   #define p521_felem_add(out, in0, in1) bignum_add_p521(out, in0, in1)
 // when s2n-bignum is used.
-#if !defined(OPENSSL_NO_ASM) && \
-    (defined(OPENSSL_LINUX) ||  defined(OPENSSL_APPLE)) && \
-    (defined(OPENSSL_X86_64) || defined(OPENSSL_AARCH64)) && \
-    !defined(MY_ASSEMBLER_IS_TOO_OLD_FOR_AVX)
+// If (1) x86_64 or aarch64, (2) linux or apple, and (3) OPENSSL_NO_ASM is not
+// set, s2n-bignum path is capable.
+#if !defined(OPENSSL_NO_ASM) &&                                                \
+    (defined(OPENSSL_LINUX) || defined(OPENSSL_APPLE)) &&                      \
+    ((defined(OPENSSL_X86_64) && !defined(MY_ASSEMBLER_IS_TOO_OLD_FOR_AVX)) || \
+     defined(OPENSSL_AARCH64))
 
 #  include "../../../third_party/s2n-bignum/include/s2n-bignum_aws-lc.h"
 #  define P521_USE_S2N_BIGNUM_FIELD_ARITH 1
@@ -98,7 +100,9 @@ static inline uint8_t p521_use_s2n_bignum_alt(void) {
 #define p521_felem_add(out, in0, in1)   bignum_add_p521(out, in0, in1)
 #define p521_felem_sub(out, in0, in1)   bignum_sub_p521(out, in0, in1)
 #define p521_felem_opp(out, in0)        bignum_neg_p521(out, in0)
+// TODO: Convert to p521_felem_to_words
 #define p521_felem_to_bytes(out, in0)   bignum_tolebytes_p521(out, in0)
+// TODO: Convert to p521_felem_from_words
 #define p521_felem_from_bytes(out, in0) bignum_fromlebytes_p521(out, in0)
 
 // The following two functions need bmi2 and adx support.
@@ -169,7 +173,9 @@ static const p521_limb_t p521_felem_p[P521_NLIMBS] = {
 #define p521_felem_opp(out, in0)        fiat_secp521r1_carry_opp(out, in0)
 #define p521_felem_mul(out, in0, in1)   fiat_secp521r1_carry_mul(out, in0, in1)
 #define p521_felem_sqr(out, in0)        fiat_secp521r1_carry_square(out, in0)
+// TODO: Convert to p521_felem_to_words
 #define p521_felem_to_bytes(out, in0)   fiat_secp521r1_to_bytes(out, in0)
+// TODO: Convert to p521_felem_from_words
 #define p521_felem_from_bytes(out, in0) fiat_secp521r1_from_bytes(out, in0)
 
 #endif // P521_USE_S2N_BIGNUM_FIELD_ARITH
@@ -214,14 +220,14 @@ static void p521_felem_cmovznz(p521_limb_t out[P521_NLIMBS],
 
 // NOTE: the input and output are in little-endian representation.
 static void p521_from_generic(p521_felem out, const EC_FELEM *in) {
-  p521_felem_from_bytes(out, in->bytes);
+  p521_felem_from_bytes(out, (const uint8_t *)in->words);
 }
 
 // NOTE: the input and output are in little-endian representation.
 static void p521_to_generic(EC_FELEM *out, const p521_felem in) {
   // |p521_felem_to_bytes| function will write the result to the first 66 bytes
   // of |out| which is exactly how many bytes are needed to represent a 521-bit
-  // element. However, EC_FELEM is a union of uint8_t array and BN_ULONG array.
+  // element.
   // The number of BN_ULONGs to represent a 521-bit value is 9 and 17, when
   // BN_ULONG is 64-bit and 32-bit, respectively. Nine 64-bit BN_ULONGs
   // translate to 72 bytes, which means that we have to make sure that the
@@ -229,7 +235,7 @@ static void p521_to_generic(EC_FELEM *out, const p521_felem in) {
   // systems and Fiat's vs. ours representation we zero out the whole element.
   OPENSSL_memset((uint8_t*)out->words, 0, sizeof(out->words));
   // Convert the element to bytes.
-  p521_felem_to_bytes(out->bytes, in);
+  p521_felem_to_bytes((uint8_t *)out->words, in);
 }
 
 // Finite field inversion using Fermat Little Theorem.
@@ -490,7 +496,7 @@ static void p521_point_add(p521_felem x3, p521_felem y3, p521_felem z3,
 // Takes the Jacobian coordinates (X, Y, Z) of a point and returns:
 //   (X', Y') = (X/Z^2, Y/Z^3).
 static int ec_GFp_nistp521_point_get_affine_coordinates(
-    const EC_GROUP *group, const EC_RAW_POINT *point,
+    const EC_GROUP *group, const EC_JACOBIAN *point,
     EC_FELEM *x_out, EC_FELEM *y_out) {
 
   if (ec_GFp_simple_is_at_infinity(group, point)) {
@@ -522,8 +528,8 @@ static int ec_GFp_nistp521_point_get_affine_coordinates(
   return 1;
 }
 
-static void ec_GFp_nistp521_add(const EC_GROUP *group, EC_RAW_POINT *r,
-                                const EC_RAW_POINT *a, const EC_RAW_POINT *b) {
+static void ec_GFp_nistp521_add(const EC_GROUP *group, EC_JACOBIAN *r,
+                                const EC_JACOBIAN *a, const EC_JACOBIAN *b) {
   p521_felem x1, y1, z1, x2, y2, z2;
   p521_from_generic(x1, &a->X);
   p521_from_generic(y1, &a->Y);
@@ -537,8 +543,8 @@ static void ec_GFp_nistp521_add(const EC_GROUP *group, EC_RAW_POINT *r,
   p521_to_generic(&r->Z, z1);
 }
 
-static void ec_GFp_nistp521_dbl(const EC_GROUP *group, EC_RAW_POINT *r,
-                                const EC_RAW_POINT *a) {
+static void ec_GFp_nistp521_dbl(const EC_GROUP *group, EC_JACOBIAN *r,
+                                const EC_JACOBIAN *a) {
   p521_felem x, y, z;
   p521_from_generic(x, &a->X);
   p521_from_generic(y, &a->Y);
@@ -581,11 +587,17 @@ static void ec_GFp_nistp521_dbl(const EC_GROUP *group, EC_RAW_POINT *r,
 // |make_tables.go| script.
 
 // p521_get_bit returns the |i|-th bit in |in|
-static crypto_word_t p521_get_bit(const uint8_t *in, int i) {
+static crypto_word_t p521_get_bit(const EC_SCALAR *in, int i) {
   if (i < 0 || i >= 521) {
     return 0;
   }
-  return (in[i >> 3] >> (i & 7)) & 1;
+#if defined(OPENSSL_64_BIT)
+  assert(sizeof(BN_ULONG) == 8);
+  return (in->words[i >> 6] >> (i & 63)) & 1;
+#else
+  assert(sizeof(BN_ULONG) == 4);
+  return (in->words[i >> 5] >> (i & 31)) & 1;
+#endif
 }
 
 // Constants for scalar encoding in the scalar multiplication functions.
@@ -599,7 +611,7 @@ OPENSSL_STATIC_ASSERT(P521_MUL_WSIZE == 5,
 #define P521_MUL_WSIZE_MASK   ((P521_MUL_TWO_TO_WSIZE << 1) - 1)
 
 // Number of |P521_MUL_WSIZE|-bit windows in a 521-bit value
-#define P521_MUL_NWINDOWS     ((521 + P521_MUL_WSIZE - 1)/P521_MUL_WSIZE) 
+#define P521_MUL_NWINDOWS     ((521 + P521_MUL_WSIZE - 1)/P521_MUL_WSIZE)
 
 // For the public point in |ec_GFp_nistp521_point_mul_public| function
 // we use window size equal to 5.
@@ -615,10 +627,10 @@ OPENSSL_STATIC_ASSERT(P521_MUL_WSIZE == 5,
 // It forces an odd scalar and outputs digits in
 // {\pm 1, \pm 3, \pm 5, \pm 7, \pm 9, ...}
 // i.e. signed odd digits with _no zeroes_ -- that makes it "regular".
-static void p521_felem_mul_scalar_rwnaf(int16_t *out, const unsigned char *in) {
+static void p521_felem_mul_scalar_rwnaf(int16_t *out, const EC_SCALAR *in) {
   int16_t window, d;
 
-  window = (in[0] & P521_MUL_WSIZE_MASK) | 1;
+  window = (in->words[0] & P521_MUL_WSIZE_MASK) | 1;
   for (size_t i = 0; i < P521_MUL_NWINDOWS - 1; i++) {
     d = (window & P521_MUL_WSIZE_MASK) - P521_MUL_TWO_TO_WSIZE;
     out[i] = d;
@@ -688,8 +700,8 @@ static void p521_select_point_affine(p521_felem out[2],
 //          negate it if s_i is negative, and add it to the accumulator.
 //
 // Note: this function is constant-time.
-static void ec_GFp_nistp521_point_mul(const EC_GROUP *group, EC_RAW_POINT *r,
-                                      const EC_RAW_POINT *p,
+static void ec_GFp_nistp521_point_mul(const EC_GROUP *group, EC_JACOBIAN *r,
+                                      const EC_JACOBIAN *p,
                                       const EC_SCALAR *scalar) {
 
   p521_felem res[3] = {{0}, {0}, {0}}, tmp[3] = {{0}, {0}, {0}}, ftmp;
@@ -717,7 +729,7 @@ static void ec_GFp_nistp521_point_mul(const EC_GROUP *group, EC_RAW_POINT *r,
 
   // Recode the scalar.
   int16_t rnaf[P521_MUL_NWINDOWS] = {0};
-  p521_felem_mul_scalar_rwnaf(rnaf, scalar->bytes);
+  p521_felem_mul_scalar_rwnaf(rnaf, scalar);
 
   // Initialize the accumulator |res| with the table entry corresponding to
   // the most significant digit of the recoded scalar (note that this digit
@@ -763,9 +775,9 @@ static void ec_GFp_nistp521_point_mul(const EC_GROUP *group, EC_RAW_POINT *r,
                  0 /* both Jacobian */, tmp[0], tmp[1], tmp[2]);
 
   // Select |res| or |tmp| based on the |scalar| parity, in constant-time.
-  p521_felem_cmovznz(res[0], scalar->bytes[0] & 1, tmp[0], res[0]);
-  p521_felem_cmovznz(res[1], scalar->bytes[0] & 1, tmp[1], res[1]);
-  p521_felem_cmovznz(res[2], scalar->bytes[0] & 1, tmp[2], res[2]);
+  p521_felem_cmovznz(res[0], scalar->words[0] & 1, tmp[0], res[0]);
+  p521_felem_cmovznz(res[1], scalar->words[0] & 1, tmp[1], res[1]);
+  p521_felem_cmovznz(res[2], scalar->words[0] & 1, tmp[2], res[2]);
 
   // Copy the result to the output.
   p521_to_generic(&r->X, res[0]);
@@ -834,14 +846,14 @@ static void ec_GFp_nistp521_point_mul(const EC_GROUP *group, EC_RAW_POINT *r,
 //
 // Note: this function is constant-time.
 static void ec_GFp_nistp521_point_mul_base(const EC_GROUP *group,
-                                           EC_RAW_POINT *r,
+                                           EC_JACOBIAN *r,
                                            const EC_SCALAR *scalar) {
 
   p521_felem res[3] = {{0}, {0}, {0}}, tmp[3] = {{0}, {0}, {0}}, ftmp;
   int16_t rnaf[P521_MUL_NWINDOWS] = {0};
 
   // Recode the scalar.
-  p521_felem_mul_scalar_rwnaf(rnaf, scalar->bytes);
+  p521_felem_mul_scalar_rwnaf(rnaf, scalar);
 
   // Process the 4 groups of digits starting from group (3) down to group (0).
   for (int i = 3; i >= 0; i--) {
@@ -897,9 +909,9 @@ static void ec_GFp_nistp521_point_mul_base(const EC_GROUP *group,
                  1 /* mixed */, tmp[0], tmp[1], p521_felem_one);
 
   // Select |res| or |tmp| based on the |scalar| parity.
-  p521_felem_cmovznz(res[0], scalar->bytes[0] & 1, tmp[0], res[0]);
-  p521_felem_cmovznz(res[1], scalar->bytes[0] & 1, tmp[1], res[1]);
-  p521_felem_cmovznz(res[2], scalar->bytes[0] & 1, tmp[2], res[2]);
+  p521_felem_cmovznz(res[0], scalar->words[0] & 1, tmp[0], res[0]);
+  p521_felem_cmovznz(res[1], scalar->words[0] & 1, tmp[1], res[1]);
+  p521_felem_cmovznz(res[2], scalar->words[0] & 1, tmp[2], res[2]);
 
   // Copy the result to the output.
   p521_to_generic(&r->X, res[0]);
@@ -941,9 +953,9 @@ static void ec_GFp_nistp521_point_mul_base(const EC_GROUP *group,
 //
 // Note: this function is NOT constant-time.
 static void ec_GFp_nistp521_point_mul_public(const EC_GROUP *group,
-                                             EC_RAW_POINT *r,
+                                             EC_JACOBIAN *r,
                                              const EC_SCALAR *g_scalar,
-                                             const EC_RAW_POINT *p,
+                                             const EC_JACOBIAN *p,
                                              const EC_SCALAR *p_scalar) {
 
   p521_felem res[3] = {{0}, {0}, {0}}, two_p[3] = {{0}, {0}, {0}}, ftmp;
