@@ -169,14 +169,19 @@ static int is_ec_fips_approved(int curve_nid) {
 
 // is_md_fips_approved_for_signing returns one if the given message digest type
 // is FIPS approved for signing, and zero otherwise.
-static int is_md_fips_approved_for_signing(int md_type) {
+static int is_md_fips_approved_for_signing(int md_type, int pkey_type) {
   switch (md_type) {
     case NID_sha224:
     case NID_sha256:
     case NID_sha384:
     case NID_sha512:
-    case NID_sha512_256:
       return 1;
+    case NID_sha512_256:
+      // SHA512/256 is only approved for signing with RSA PSS
+      if (pkey_type == EVP_PKEY_RSA_PSS) {
+        return 1;
+      }
+      return 0;
     default:
       return 0;
   }
@@ -184,15 +189,20 @@ static int is_md_fips_approved_for_signing(int md_type) {
 
 // is_md_fips_approved_for_verifying returns one if the given message digest
 // type is FIPS approved for verifying, and zero otherwise.
-static int is_md_fips_approved_for_verifying(int md_type) {
+static int is_md_fips_approved_for_verifying(int md_type, int pkey_type) {
   switch (md_type) {
     case NID_sha1:
     case NID_sha224:
     case NID_sha256:
     case NID_sha384:
     case NID_sha512:
-    case NID_sha512_256:
       return 1;
+    case NID_sha512_256:
+      // SHA512/256 is only approved for verifying with RSA PSS
+      if (pkey_type == EVP_PKEY_RSA_PSS) {
+        return 1;
+      }
+      return 0;
     default:
       return 0;
   }
@@ -200,7 +210,7 @@ static int is_md_fips_approved_for_verifying(int md_type) {
 
 static void evp_md_ctx_verify_service_indicator(const EVP_MD_CTX *ctx,
                                                 int rsa_1024_ok,
-                                                int (*md_ok)(int md_type)) {
+                                                int (*md_ok)(int md_type, int pkey_type)) {
   if (EVP_MD_CTX_md(ctx) == NULL) {
     // Signature schemes without a prehash are currently never FIPS approved.
     goto err;
@@ -245,7 +255,7 @@ static void evp_md_ctx_verify_service_indicator(const EVP_MD_CTX *ctx,
     size_t pkey_size = EVP_PKEY_size(ctx->pctx->pkey);
 
     // Check if the MD type and the RSA key size are approved.
-    if (md_ok(md_type) &&
+    if (md_ok(md_type, pkey_type) &&
         ((rsa_1024_ok && pkey_size == 128) || pkey_size == 256 ||
          pkey_size == 384 || pkey_size == 512)) {
       FIPS_service_indicator_update_state();
@@ -253,7 +263,7 @@ static void evp_md_ctx_verify_service_indicator(const EVP_MD_CTX *ctx,
   } else if (pkey_type == EVP_PKEY_EC) {
     // Check if the MD type and the elliptic curve are approved.
     int curve_nid = EC_GROUP_get_curve_name(pkey->pkey.ec->group);
-    if (md_ok(md_type) && is_ec_fips_approved(curve_nid)) {
+    if (md_ok(md_type, pkey_type) && is_ec_fips_approved(curve_nid)) {
       FIPS_service_indicator_update_state();
     }
   }
