@@ -75,9 +75,7 @@
 #include <openssl/is_awslc.h>
 #include <openssl/opensslconf.h>
 
-#if defined(BORINGSSL_PREFIX)
-#include <boringssl_prefix_symbols.h>
-#endif
+#include <openssl/boringssl_prefix_symbols.h>
 
 #if defined(__cplusplus)
 extern "C" {
@@ -113,6 +111,9 @@ extern "C" {
 #define OPENSSL_RISCV64
 #elif defined(__riscv) && __SIZEOF_POINTER__ == 4
 #define OPENSSL_32_BIT
+#elif defined(__loongarch_lp64)
+#define OPENSSL_64_BIT
+#define OPENSSL_LOONGARCH64	
 #elif defined(__pnacl__)
 #define OPENSSL_32_BIT
 #define OPENSSL_PNACL
@@ -216,7 +217,7 @@ extern "C" {
 // ServiceIndicatorTest.AWSLCVersionString
 // Note: there are two versions of this test. Only one test is compiled
 // depending on FIPS mode.
-#define AWSLC_VERSION_NUMBER_STRING "1.12.1"
+#define AWSLC_VERSION_NUMBER_STRING "1.16.0"
 
 #if defined(BORINGSSL_SHARED_LIBRARY)
 
