@@ -39,7 +39,7 @@ impl Harness {
 }
 
 impl super::Connection for Harness {
-    fn poll<Ctx: Context>(&mut self, context: &mut Ctx) -> Poll<Result<()>> {
+    fn poll_negotiate<Ctx: Context>(&mut self, context: &mut Ctx) -> Poll<Result<()>> {
         let mut callback: Callback<Ctx> = Callback {
             context,
             err: None,
@@ -67,6 +67,33 @@ impl super::Connection for Harness {
             Poll::Pending => Poll::Pending,
         }
     }
+
+    fn poll_action<Ctx: Context, F>(&mut self, context: &mut Ctx, action: F) -> Poll<Result<()>>
+    where
+        F: FnOnce(&mut Connection) -> Poll<Result<usize, crate::error::Error>>,
+    {
+        let mut callback: Callback<Ctx> = Callback {
+            context,
+            err: None,
+            send_buffer: &mut self.send_buffer,
+        };
+
+        unsafe {
+            // Safety: the callback struct must live as long as the callbacks are
+            // set on on the connection
+            callback.set(&mut self.connection);
+        }
+
+        let result = action(&mut self.connection);
+
+        callback.unset(&mut self.connection)?;
+
+        match result {
+            Poll::Ready(Ok(_)) => Ok(()).into(),
+            Poll::Ready(Err(err)) => Err(err.into()).into(),
+            Poll::Pending => Poll::Pending,
+        }
+    }
 }
 
 struct Callback<'a, T> {
@@ -309,10 +336,9 @@ mod tests {
     #[test]
     fn failing_client_hello_callback_sync() -> Result<(), Error> {
         let (waker, wake_count) = new_count_waker();
-        let handle = FailingCHHandler::default();
         let config = {
             let mut config = config_builder(&security::DEFAULT_TLS13)?;
-            config.set_client_hello_callback(handle)?;
+            config.set_client_hello_callback(FailingCHHandler)?;
             config.build()?
         };
 
@@ -359,10 +385,9 @@ mod tests {
     #[test]
     fn failing_client_hello_callback_async() -> Result<(), Error> {
         let (waker, wake_count) = new_count_waker();
-        let handle = FailingAsyncCHHandler::default();
         let config = {
             let mut config = config_builder(&security::DEFAULT_TLS13)?;
-            config.set_client_hello_callback(handle)?;
+            config.set_client_hello_callback(FailingAsyncCHHandler)?;
             config.build()?
         };
 
