@@ -28,6 +28,8 @@
 
 static int s2n_signature_scheme_valid_to_offer(struct s2n_connection *conn, const struct s2n_signature_scheme *scheme)
 {
+    POSIX_ENSURE_REF(conn);
+
     /* We don't know what protocol version we will eventually negotiate, but we know that it won't be any higher. */
     POSIX_ENSURE_GTE(conn->actual_protocol_version, scheme->minimum_protocol_version);
 
@@ -50,6 +52,7 @@ static int s2n_signature_scheme_valid_to_offer(struct s2n_connection *conn, cons
 static int s2n_signature_scheme_valid_to_accept(struct s2n_connection *conn, const struct s2n_signature_scheme *scheme)
 {
     POSIX_ENSURE_REF(scheme);
+    POSIX_ENSURE_REF(conn);
 
     POSIX_GUARD(s2n_signature_scheme_valid_to_offer(conn, scheme));
 
@@ -57,6 +60,14 @@ static int s2n_signature_scheme_valid_to_accept(struct s2n_connection *conn, con
         POSIX_ENSURE_LTE(conn->actual_protocol_version, scheme->maximum_protocol_version);
     }
 
+    POSIX_ENSURE_NE(conn->actual_protocol_version, S2N_UNKNOWN_PROTOCOL_VERSION);
+    if (conn->actual_protocol_version >= S2N_TLS13) {
+        POSIX_ENSURE_NE(scheme->hash_alg, S2N_HASH_SHA1);
+        POSIX_ENSURE_NE(scheme->sig_alg, S2N_SIGNATURE_RSA);
+    } else {
+        POSIX_ENSURE_NE(scheme->sig_alg, S2N_SIGNATURE_RSA_PSS_PSS);
+    }
+
     return 0;
 }
 
@@ -72,7 +83,7 @@ static int s2n_is_signature_scheme_usable(struct s2n_connection *conn, const str
 }
 
 static int s2n_choose_sig_scheme(struct s2n_connection *conn, struct s2n_sig_scheme_list *peer_wire_prefs,
-        struct s2n_signature_scheme *chosen_scheme_out)
+        const struct s2n_signature_scheme **chosen_scheme_out)
 {
     POSIX_ENSURE_REF(conn);
     POSIX_ENSURE_REF(conn->secure);
@@ -94,7 +105,7 @@ static int s2n_choose_sig_scheme(struct s2n_connection *conn, struct s2n_sig_sch
             uint16_t their_iana_val = peer_wire_prefs->iana_list[j];
 
             if (candidate->iana_value == their_iana_val) {
-                *chosen_scheme_out = *candidate;
+                *chosen_scheme_out = candidate;
                 return S2N_SUCCESS;
             }
         }
@@ -105,7 +116,8 @@ static int s2n_choose_sig_scheme(struct s2n_connection *conn, struct s2n_sig_sch
 }
 
 /* similar to s2n_choose_sig_scheme() without matching client's preference */
-int s2n_tls13_default_sig_scheme(struct s2n_connection *conn, struct s2n_signature_scheme *chosen_scheme_out)
+int s2n_tls13_default_sig_scheme(struct s2n_connection *conn,
+        const struct s2n_signature_scheme **chosen_scheme_out)
 {
     POSIX_ENSURE_REF(conn);
     POSIX_ENSURE_REF(conn->secure);
@@ -124,7 +136,7 @@ int s2n_tls13_default_sig_scheme(struct s2n_connection *conn, struct s2n_signatu
             continue;
         }
 
-        *chosen_scheme_out = *candidate;
+        *chosen_scheme_out = candidate;
         return S2N_SUCCESS;
     }
 
@@ -132,7 +144,7 @@ int s2n_tls13_default_sig_scheme(struct s2n_connection *conn, struct s2n_signatu
 }
 
 int s2n_get_and_validate_negotiated_signature_scheme(struct s2n_connection *conn, struct s2n_stuffer *in,
-        struct s2n_signature_scheme *chosen_sig_scheme)
+        const struct s2n_signature_scheme **chosen_sig_scheme)
 {
     uint16_t actual_iana_val;
     POSIX_GUARD(s2n_stuffer_read_uint16(in, &actual_iana_val));
@@ -149,28 +161,16 @@ int s2n_get_and_validate_negotiated_signature_scheme(struct s2n_connection *conn
         }
 
         if (candidate->iana_value == actual_iana_val) {
-            *chosen_sig_scheme = *candidate;
+            *chosen_sig_scheme = candidate;
             return S2N_SUCCESS;
         }
     }
 
-    /* We require an exact match in TLS 1.3, but all previous versions can fall back to the default SignatureScheme.
-     * This means that an s2n client will accept the default SignatureScheme from a TLS server, even if the client did
-     * not send it in it's ClientHello. This pre-TLS1.3 behavior is an intentional choice to maximize support. */
-    struct s2n_signature_scheme default_scheme = { 0 };
-    POSIX_GUARD(s2n_choose_default_sig_scheme(conn, &default_scheme, S2N_PEER_MODE(conn->mode)));
-
-    if ((conn->actual_protocol_version <= S2N_TLS12)
-            && (s2n_signature_scheme_valid_to_accept(conn, &default_scheme) == S2N_SUCCESS)
-            && (actual_iana_val == default_scheme.iana_value)) {
-        *chosen_sig_scheme = default_scheme;
-        return S2N_SUCCESS;
-    }
-
     POSIX_BAIL(S2N_ERR_INVALID_SIGNATURE_SCHEME);
 }
 
-int s2n_choose_default_sig_scheme(struct s2n_connection *conn, struct s2n_signature_scheme *sig_scheme_out, s2n_mode signer)
+int s2n_choose_default_sig_scheme(struct s2n_connection *conn,
+        const struct s2n_signature_scheme **sig_scheme_out, s2n_mode signer)
 {
     POSIX_ENSURE_REF(conn);
     POSIX_ENSURE_REF(conn->secure);
@@ -187,24 +187,48 @@ int s2n_choose_default_sig_scheme(struct s2n_connection *conn, struct s2n_signat
     /* Default our signature digest algorithms.
      * For >=TLS 1.2 this default may be overridden by the signature_algorithms extension.
      */
+    const struct s2n_signature_scheme *default_sig_scheme = &s2n_rsa_pkcs1_md5_sha1;
     if (auth_method == S2N_AUTHENTICATION_ECDSA) {
-        *sig_scheme_out = s2n_ecdsa_sha1;
+        default_sig_scheme = &s2n_ecdsa_sha1;
     } else if (conn->actual_protocol_version >= S2N_TLS12) {
-        *sig_scheme_out = s2n_rsa_pkcs1_sha1;
-    } else {
-        *sig_scheme_out = s2n_rsa_pkcs1_md5_sha1;
+        default_sig_scheme = &s2n_rsa_pkcs1_sha1;
     }
 
-    return S2N_SUCCESS;
+    if (conn->actual_protocol_version < S2N_TLS12) {
+        /* Before TLS1.2, signature algorithms were fixed, not chosen / negotiated. */
+        *sig_scheme_out = default_sig_scheme;
+        return S2N_SUCCESS;
+    } else {
+        /* If we attempt to negotiate a default in TLS1.2, we should ensure that
+         * default is allowed by the local security policy.
+         */
+        const struct s2n_signature_preferences *signature_preferences = NULL;
+        POSIX_GUARD(s2n_connection_get_signature_preferences(conn, &signature_preferences));
+        POSIX_ENSURE_REF(signature_preferences);
+        for (size_t i = 0; i < signature_preferences->count; i++) {
+            if (signature_preferences->signature_schemes[i]->iana_value == default_sig_scheme->iana_value) {
+                *sig_scheme_out = default_sig_scheme;
+                return S2N_SUCCESS;
+            }
+        }
+        /* We cannot bail with an error here because existing logic assumes
+         * that this method should always succeed and calls it even when no default
+         * is actually necessary.
+         * If no valid default exists, set an unusable, invalid empty scheme.
+         */
+        *sig_scheme_out = &s2n_null_sig_scheme;
+        return S2N_SUCCESS;
+    }
 }
 
-int s2n_choose_sig_scheme_from_peer_preference_list(struct s2n_connection *conn, struct s2n_sig_scheme_list *peer_wire_prefs,
-        struct s2n_signature_scheme *sig_scheme_out)
+int s2n_choose_sig_scheme_from_peer_preference_list(struct s2n_connection *conn,
+        struct s2n_sig_scheme_list *peer_wire_prefs,
+        const struct s2n_signature_scheme **sig_scheme_out)
 {
     POSIX_ENSURE_REF(conn);
     POSIX_ENSURE_REF(sig_scheme_out);
 
-    struct s2n_signature_scheme chosen_scheme = { 0 };
+    const struct s2n_signature_scheme *chosen_scheme = &s2n_null_sig_scheme;
     if (conn->actual_protocol_version < S2N_TLS13) {
         POSIX_GUARD(s2n_choose_default_sig_scheme(conn, &chosen_scheme, conn->mode));
     } else {
@@ -283,7 +307,7 @@ int s2n_recv_supported_sig_scheme_list(struct s2n_stuffer *in, struct s2n_sig_sc
 
     sig_hash_algs->len = 0;
 
-    for (size_t i = 0; i < pairs_available; i++) {
+    for (size_t i = 0; i < (size_t) pairs_available; i++) {
         uint16_t sig_scheme = 0;
         POSIX_GUARD(s2n_stuffer_read_uint16(in, &sig_scheme));
 
